@@ -1,8 +1,28 @@
+/* screenshot-save.c - image saving functions for GNOME Screenshot
+ *
+ * Copyright (C) 2001-2006  Jonathan Blandford <jrb@alum.mit.edu>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ */
+
 #include <config.h>
 #include <gnome.h>
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "screenshot-save.h"
@@ -13,10 +33,6 @@ static char *tmp_filename = NULL;
 static SaveFunction save_callback = NULL;
 static gpointer save_user_data = NULL;
 
-extern int errno;
-
-
-
 /* Strategy for saving:
  *
  * We keep another process around to handle saving the image.  This is
@@ -38,7 +54,7 @@ clean_up_temporary_dir (gboolean gui_on_error)
 
   if (error_occurred)
     {
-      message = g_strdup_printf (_("Unable to clear the temporary directory:\n%s"),
+      message = g_strdup_printf (_("Unable to clear the temporary folder:\n%s"),
 				 tmp_filename);
       if (gui_on_error)
 	{
@@ -95,16 +111,21 @@ read_pipe_from_child (GIOChannel   *source,
       gchar *message = NULL;
       gchar *error_message = NULL;
       GtkWidget *dialog;
+      GIOStatus status;
 
-      g_io_channel_read_line (source, &error_message, NULL, NULL, NULL);
-      message = g_strdup_printf ("Unable to save the screenshot to disk:\n\n%s", error_message);
-      dialog = gtk_message_dialog_new (NULL, 0,
-				       GTK_MESSAGE_ERROR,
-				       GTK_BUTTONS_OK,
-				       message);
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      exit (1);
+      status = g_io_channel_read_line (source, &error_message, NULL, NULL, NULL);
+
+      if (status == G_IO_STATUS_NORMAL)
+	{
+	  message = g_strdup_printf ("Unable to save the screenshot to disk:\n\n%s", error_message);
+	  dialog = gtk_message_dialog_new (NULL, 0,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   message);
+	  gtk_dialog_run (GTK_DIALOG (dialog));
+	  gtk_widget_destroy (dialog);
+	  exit (1);
+	}
     }
 
   (*save_callback) (save_user_data);
@@ -115,22 +136,34 @@ read_pipe_from_child (GIOChannel   *source,
 static char *
 make_temp_directory (void)
 {
-  gint result;
-  char *dir_name;
-
+  gint result, i;
+  gchar *dir_name;
+  
+  i = 0;
   do
     {
-      /* FIXME: honor $TMPDIR */
-      dir_name = g_strdup_printf ("/tmp/gnome-panel-screenshot-%d", rand ());
-      result = mkdir (dir_name, 0700);
+      gchar *tmp_dir = g_strdup_printf ("%u-%d",
+                                        (unsigned int) getpid (),
+                                        i++);
+      
+      dir_name = g_build_filename (g_get_tmp_dir (),
+                                   "gnome-screenshot",
+                                   tmp_dir,
+                                   NULL);
+      g_free (tmp_dir);
 
-      if (result == 0)
-	return dir_name;
-      g_free (dir_name);
+      result = g_mkdir_with_parents (dir_name, 0700);
+      if (result < 0)
+        {
+          g_free (dir_name);
 
-      /* If a dir of that name already exists, we try again.  Otherwise, we give up.  */
-      if (errno != EEXIST)
-	return NULL;
+          if (errno != EEXIST)
+            return NULL;
+          else
+            continue;
+        }
+      else
+        return dir_name;
     }
   while (TRUE);
 }
